@@ -6,9 +6,8 @@ import (
 	"fmt"
 	"io"
 
-	"github.com/smira/aptly/aptly"
-	"github.com/smira/aptly/database"
-	"github.com/smira/go-uuid/uuid"
+	"github.com/aptly-dev/aptly/database"
+	"github.com/google/uuid"
 )
 
 // ContentsIndex calculates mapping from files to packages, with sorting and aggregation
@@ -21,19 +20,16 @@ type ContentsIndex struct {
 func NewContentsIndex(db database.Storage) *ContentsIndex {
 	return &ContentsIndex{
 		db:     db,
-		prefix: []byte(uuid.New()),
+		prefix: []byte(uuid.NewString()),
 	}
 }
 
 // Push adds package to contents index, calculating package contents as required
-func (index *ContentsIndex) Push(p *Package, packagePool aptly.PackagePool, progress aptly.Progress) error {
-	contents := p.Contents(packagePool, progress)
-	qualifiedName := []byte(p.QualifiedName())
-
+func (index *ContentsIndex) Push(qualifiedName []byte, contents []string, dbw database.Writer) error {
 	for _, path := range contents {
 		// for performance reasons we only write to leveldb during push.
 		// merging of qualified names per path will be done in WriteTo
-		err := index.db.Put(append(append(append(index.prefix, []byte(path)...), byte(0)), qualifiedName...), nil)
+		err := dbw.Put(append(append(append(index.prefix, []byte(path)...), byte(0)), qualifiedName...), nil)
 		if err != nil {
 			return err
 		}
@@ -68,7 +64,7 @@ func (index *ContentsIndex) WriteTo(w io.Writer) (int64, error) {
 		currentPkgs [][]byte
 	)
 
-	err = index.db.ProcessByPrefix(index.prefix, func(key []byte, value []byte) error {
+	err = index.db.ProcessByPrefix(index.prefix, func(key []byte, _ []byte) error {
 		// cut prefix
 		key = key[prefixLen:]
 
