@@ -6,8 +6,8 @@ import (
 	"os"
 
 	"github.com/DisposaBoy/JsonConfigReader"
-	"github.com/smira/aptly/pgp"
-	"github.com/smira/aptly/utils"
+	"github.com/aptly-dev/aptly/pgp"
+	"github.com/aptly-dev/aptly/utils"
 )
 
 // UploadersRule is single rule of format: what packages can group or key upload
@@ -41,7 +41,7 @@ func NewUploadersFromFile(path string) (*Uploaders, error) {
 	if err != nil {
 		return nil, fmt.Errorf("error loading uploaders file: %s", err)
 	}
-	defer f.Close()
+	defer func() { _ = f.Close() }()
 
 	err = json.NewDecoder(JsonConfigReader.New(f)).Decode(&uploaders)
 	if err != nil {
