@@ -3,16 +3,17 @@ package files
 import (
 	"fmt"
 	"io"
-	"io/ioutil"
 	"os"
 	"path/filepath"
+	"sort"
 	"sync"
 	"syscall"
 
-	"github.com/smira/go-uuid/uuid"
+	"github.com/google/uuid"
+	"github.com/saracen/walker"
 
-	"github.com/smira/aptly/aptly"
-	"github.com/smira/aptly/utils"
+	"github.com/aptly-dev/aptly/aptly"
+	"github.com/aptly-dev/aptly/utils"
 )
 
 // PackagePool is deduplicated storage of package files on filesystem
@@ -31,8 +32,7 @@ var (
 
 // NewPackagePool creates new instance of PackagePool which specified root
 func NewPackagePool(root string, supportLegacyPaths bool) *PackagePool {
-	rootPath := filepath.Join(root, "pool")
-	rootPath, err := filepath.Abs(rootPath)
+	rootPath, err := filepath.Abs(root)
 	if err != nil {
 		panic(err)
 	}
@@ -81,7 +81,7 @@ func (pool *PackagePool) FilepathList(progress aptly.Progress) ([]string, error)
 	pool.Lock()
 	defer pool.Unlock()
 
-	dirs, err := ioutil.ReadDir(pool.rootPath)
+	dirs, err := os.ReadDir(pool.rootPath)
 	if err != nil {
 		if os.IsNotExist(err) {
 			return nil, nil
@@ -94,18 +94,18 @@ func (pool *PackagePool) FilepathList(progress aptly.Progress) ([]string, error)
 	}
 
 	if progress != nil {
-		progress.InitBar(int64(len(dirs)), false)
+		progress.InitBar(int64(len(dirs)), false, aptly.BarGeneralBuildFileList)
 		defer progress.ShutdownBar()
 	}
 
 	result := []string{}
+	resultLock := &sync.Mutex{}
 
 	for _, dir := range dirs {
-		err = filepath.Walk(filepath.Join(pool.rootPath, dir.Name()), func(path string, info os.FileInfo, err error) error {
-			if err != nil {
-				return err
-			}
+		err = walker.Walk(filepath.Join(pool.rootPath, dir.Name()), func(path string, info os.FileInfo) error {
 			if !info.IsDir() {
+				resultLock.Lock()
+				defer resultLock.Unlock()
 				result = append(result, path[len(pool.rootPath)+1:])
 			}
 			return nil
@@ -119,6 +119,7 @@ func (pool *PackagePool) FilepathList(progress aptly.Progress) ([]string, error)
 		}
 	}
 
+	sort.Strings(result)
 	return result, nil
 }
 
@@ -239,7 +240,9 @@ func (pool *PackagePool) Import(srcPath, basename string, checksums *utils.Check
 	if err != nil {
 		return "", err
 	}
-	defer source.Close()
+	defer func() {
+            _ = source.Close()
+	}()
 
 	sourceInfo, err := source.Stat()
 	if err != nil {
@@ -350,7 +353,9 @@ func (pool *PackagePool) Import(srcPath, basename string, checksums *utils.Check
 		if err != nil {
 			return "", err
 		}
-		defer target.Close()
+		defer func() {
+			_ = target.Close()
+		}()
 
 		_, err = io.Copy(target, source)
 
@@ -378,6 +383,15 @@ func (pool *PackagePool) Import(srcPath, basename string, checksums *utils.Check
 	return poolPath, err
 }
 
+func (pool *PackagePool) Size(path string) (size int64, err error) {
+	stat, err := pool.Stat(path)
+	if err != nil {
+		return 0, err
+	}
+
+	return stat.Size(), nil
+}
+
 // Open returns io.ReadCloser to access the file
 func (pool *PackagePool) Open(path string) (aptly.ReadSeekerCloser, error) {
 	return os.Open(filepath.Join(pool.rootPath, path))
@@ -407,7 +421,7 @@ func (pool *PackagePool) FullPath(path string) string {
 
 // GenerateTempPath generates temporary path for download (which is fast to import into package pool later on)
 func (pool *PackagePool) GenerateTempPath(filename string) (string, error) {
-	random := uuid.NewRandom().String()
+	random := uuid.NewString()
 
 	return filepath.Join(pool.rootPath, random[0:2], random[2:4], random[4:]+filename), nil
 }
