@@ -2,14 +2,13 @@ package files
 
 import (
 	"io"
-	"io/ioutil"
 	"os"
 	"path/filepath"
 	"runtime"
 	"syscall"
 
-	"github.com/smira/aptly/aptly"
-	"github.com/smira/aptly/utils"
+	"github.com/aptly-dev/aptly/aptly"
+	"github.com/aptly-dev/aptly/utils"
 
 	. "gopkg.in/check.v1"
 )
@@ -51,18 +50,18 @@ func (s *PackagePoolSuite) TestFilepathList(c *C) {
 	c.Check(err, IsNil)
 	c.Check(list, IsNil)
 
-	os.MkdirAll(filepath.Join(s.pool.rootPath, "bd", "0b"), 0755)
-	os.MkdirAll(filepath.Join(s.pool.rootPath, "bd", "0a"), 0755)
-	os.MkdirAll(filepath.Join(s.pool.rootPath, "ae", "0c"), 0755)
+	_ = os.MkdirAll(filepath.Join(s.pool.rootPath, "bd", "0b"), 0755)
+	_ = os.MkdirAll(filepath.Join(s.pool.rootPath, "bd", "0a"), 0755)
+	_ = os.MkdirAll(filepath.Join(s.pool.rootPath, "ae", "0c"), 0755)
 
 	list, err = s.pool.FilepathList(nil)
 	c.Check(err, IsNil)
 	c.Check(list, DeepEquals, []string{})
 
-	ioutil.WriteFile(filepath.Join(s.pool.rootPath, "ae", "0c", "1.deb"), nil, 0644)
-	ioutil.WriteFile(filepath.Join(s.pool.rootPath, "ae", "0c", "2.deb"), nil, 0644)
-	ioutil.WriteFile(filepath.Join(s.pool.rootPath, "bd", "0a", "3.deb"), nil, 0644)
-	ioutil.WriteFile(filepath.Join(s.pool.rootPath, "bd", "0b", "4.deb"), nil, 0644)
+	_ = os.WriteFile(filepath.Join(s.pool.rootPath, "ae", "0c", "1.deb"), nil, 0644)
+	_ = os.WriteFile(filepath.Join(s.pool.rootPath, "ae", "0c", "2.deb"), nil, 0644)
+	_ = os.WriteFile(filepath.Join(s.pool.rootPath, "bd", "0a", "3.deb"), nil, 0644)
+	_ = os.WriteFile(filepath.Join(s.pool.rootPath, "bd", "0b", "4.deb"), nil, 0644)
 
 	list, err = s.pool.FilepathList(nil)
 	c.Check(err, IsNil)
@@ -70,14 +69,14 @@ func (s *PackagePoolSuite) TestFilepathList(c *C) {
 }
 
 func (s *PackagePoolSuite) TestRemove(c *C) {
-	os.MkdirAll(filepath.Join(s.pool.rootPath, "bd", "0b"), 0755)
-	os.MkdirAll(filepath.Join(s.pool.rootPath, "bd", "0a"), 0755)
-	os.MkdirAll(filepath.Join(s.pool.rootPath, "ae", "0c"), 0755)
+	_ = os.MkdirAll(filepath.Join(s.pool.rootPath, "bd", "0b"), 0755)
+	_ = os.MkdirAll(filepath.Join(s.pool.rootPath, "bd", "0a"), 0755)
+	_ = os.MkdirAll(filepath.Join(s.pool.rootPath, "ae", "0c"), 0755)
 
-	ioutil.WriteFile(filepath.Join(s.pool.rootPath, "ae", "0c", "1.deb"), []byte("1"), 0644)
-	ioutil.WriteFile(filepath.Join(s.pool.rootPath, "ae", "0c", "2.deb"), []byte("22"), 0644)
-	ioutil.WriteFile(filepath.Join(s.pool.rootPath, "bd", "0a", "3.deb"), []byte("333"), 0644)
-	ioutil.WriteFile(filepath.Join(s.pool.rootPath, "bd", "0b", "4.deb"), []byte("4444"), 0644)
+	_ = os.WriteFile(filepath.Join(s.pool.rootPath, "ae", "0c", "1.deb"), []byte("1"), 0644)
+	_ = os.WriteFile(filepath.Join(s.pool.rootPath, "ae", "0c", "2.deb"), []byte("22"), 0644)
+	_ = os.WriteFile(filepath.Join(s.pool.rootPath, "bd", "0a", "3.deb"), []byte("333"), 0644)
+	_ = os.WriteFile(filepath.Join(s.pool.rootPath, "bd", "0b", "4.deb"), []byte("4444"), 0644)
 
 	size, err := s.pool.Remove("ae/0c/2.deb")
 	c.Check(err, IsNil)
@@ -91,6 +90,21 @@ func (s *PackagePoolSuite) TestRemove(c *C) {
 	c.Check(list, DeepEquals, []string{"ae/0c/1.deb", "bd/0a/3.deb", "bd/0b/4.deb"})
 }
 
+func isSameDevice(s *PackagePoolSuite) bool {
+	poolPath, _ := s.pool.buildPoolPath(filepath.Base(s.debFile), &s.checksum)
+	fullPoolPath := filepath.Join(s.pool.rootPath, poolPath)
+	poolDir := filepath.Dir(fullPoolPath)
+	poolDirInfo, _ := os.Stat(poolDir)
+
+	source, _ := os.Open(s.debFile)
+	sourceInfo, _ := source.Stat()
+	defer func() {
+		_ = source.Close()
+	}()
+
+	return poolDirInfo.Sys().(*syscall.Stat_t).Dev == sourceInfo.Sys().(*syscall.Stat_t).Dev
+}
+
 func (s *PackagePoolSuite) TestImportOk(c *C) {
 	path, err := s.pool.Import(s.debFile, filepath.Base(s.debFile), &s.checksum, false, s.cs)
 	c.Check(err, IsNil)
@@ -98,19 +112,24 @@ func (s *PackagePoolSuite) TestImportOk(c *C) {
 	// SHA256 should be automatically calculated
 	c.Check(s.checksum.SHA256, Equals, "c76b4bd12fd92e4dfe1b55b18a67a669d92f62985d6a96c8a21d96120982cf12")
 	// checksum storage is filled with new checksum
-	c.Check(s.cs.(*mockChecksumStorage).store[path].SHA256, Equals, "c76b4bd12fd92e4dfe1b55b18a67a669d92f62985d6a96c8a21d96120982cf12")
+	c.Check(s.cs.(*MockChecksumStorage).Store[path].SHA256, Equals, "c76b4bd12fd92e4dfe1b55b18a67a669d92f62985d6a96c8a21d96120982cf12")
 
 	info, err := s.pool.Stat(path)
 	c.Assert(err, IsNil)
 	c.Check(info.Size(), Equals, int64(2738))
-	c.Check(info.Sys().(*syscall.Stat_t).Nlink > 1, Equals, true)
+	// /tmp may be on different devices, so hardlinks are not used
+	if isSameDevice(s) {
+		c.Check(info.Sys().(*syscall.Stat_t).Nlink > 1, Equals, true)
+	} else {
+		c.Check(info.Sys().(*syscall.Stat_t).Nlink == 1, Equals, true)
+	}
 
 	// import as different name
 	path, err = s.pool.Import(s.debFile, "some.deb", &s.checksum, false, s.cs)
 	c.Check(err, IsNil)
 	c.Check(path, Equals, "c7/6b/4bd12fd92e4dfe1b55b18a67a669_some.deb")
 	// checksum storage is filled with new checksum
-	c.Check(s.cs.(*mockChecksumStorage).store[path].SHA256, Equals, "c76b4bd12fd92e4dfe1b55b18a67a669d92f62985d6a96c8a21d96120982cf12")
+	c.Check(s.cs.(*MockChecksumStorage).Store[path].SHA256, Equals, "c76b4bd12fd92e4dfe1b55b18a67a669d92f62985d6a96c8a21d96120982cf12")
 
 	// double import, should be ok
 	s.checksum.SHA512 = "" // clear checksum
@@ -121,7 +140,7 @@ func (s *PackagePoolSuite) TestImportOk(c *C) {
 	c.Check(s.checksum.SHA512, Equals, "d7302241373da972aa9b9e71d2fd769b31a38f71182aa71bc0d69d090d452c69bb74b8612c002ccf8a89c279ced84ac27177c8b92d20f00023b3d268e6cec69c")
 
 	// clear checksum storage, and do double-import
-	delete(s.cs.(*mockChecksumStorage).store, path)
+	delete(s.cs.(*MockChecksumStorage).Store, path)
 	s.checksum.SHA512 = "" // clear checksum
 	path, err = s.pool.Import(s.debFile, filepath.Base(s.debFile), &s.checksum, false, s.cs)
 	c.Check(err, IsNil)
@@ -139,7 +158,7 @@ func (s *PackagePoolSuite) TestImportOk(c *C) {
 }
 
 func (s *PackagePoolSuite) TestImportLegacy(c *C) {
-	os.MkdirAll(filepath.Join(s.pool.rootPath, "00", "35"), 0755)
+	_ = os.MkdirAll(filepath.Join(s.pool.rootPath, "00", "35"), 0755)
 	err := utils.CopyFile(s.debFile, filepath.Join(s.pool.rootPath, "00", "35", "libboost-program-options-dev_1.49.0.1_i386.deb"))
 	c.Assert(err, IsNil)
 
@@ -160,7 +179,7 @@ func (s *PackagePoolSuite) TestVerifyLegacy(c *C) {
 	c.Check(err, IsNil)
 	c.Check(exists, Equals, false)
 
-	os.MkdirAll(filepath.Join(s.pool.rootPath, "00", "35"), 0755)
+	_ = os.MkdirAll(filepath.Join(s.pool.rootPath, "00", "35"), 0755)
 	err = utils.CopyFile(s.debFile, filepath.Join(s.pool.rootPath, "00", "35", "libboost-program-options-dev_1.49.0.1_i386.deb"))
 	c.Assert(err, IsNil)
 
@@ -226,7 +245,7 @@ func (s *PackagePoolSuite) TestVerify(c *C) {
 	c.Check(exists, Equals, false)
 
 	// check existence, with missing checksum and no info in checksum storage
-	delete(s.cs.(*mockChecksumStorage).store, path)
+	delete(s.cs.(*MockChecksumStorage).Store, path)
 	s.checksum.SHA512 = ""
 	ppath, exists, err = s.pool.Verify("", filepath.Base(s.debFile), &s.checksum, s.cs)
 	c.Check(ppath, Equals, path)
@@ -281,13 +300,25 @@ func (s *PackagePoolSuite) TestImportNotExist(c *C) {
 }
 
 func (s *PackagePoolSuite) TestImportOverwrite(c *C) {
-	os.MkdirAll(filepath.Join(s.pool.rootPath, "c7", "6b"), 0755)
-	ioutil.WriteFile(filepath.Join(s.pool.rootPath, "c7", "6b", "4bd12fd92e4dfe1b55b18a67a669_libboost-program-options-dev_1.49.0.1_i386.deb"), []byte("1"), 0644)
+	_ = os.MkdirAll(filepath.Join(s.pool.rootPath, "c7", "6b"), 0755)
+	_ = os.WriteFile(filepath.Join(s.pool.rootPath, "c7", "6b", "4bd12fd92e4dfe1b55b18a67a669_libboost-program-options-dev_1.49.0.1_i386.deb"), []byte("1"), 0644)
 
 	_, err := s.pool.Import(s.debFile, filepath.Base(s.debFile), &s.checksum, false, s.cs)
 	c.Check(err, ErrorMatches, "unable to import into pool.*")
 }
 
+func (s *PackagePoolSuite) TestSize(c *C) {
+	path, err := s.pool.Import(s.debFile, filepath.Base(s.debFile), &s.checksum, false, s.cs)
+	c.Check(err, IsNil)
+
+	size, err := s.pool.Size(path)
+	c.Assert(err, IsNil)
+	c.Check(size, Equals, int64(2738))
+
+	_, err = s.pool.Size("do/es/ntexist")
+	c.Assert(os.IsNotExist(err), Equals, true)
+}
+
 func (s *PackagePoolSuite) TestStat(c *C) {
 	path, err := s.pool.Import(s.debFile, filepath.Base(s.debFile), &s.checksum, false, s.cs)
 	c.Check(err, IsNil)
@@ -306,7 +337,7 @@ func (s *PackagePoolSuite) TestOpen(c *C) {
 
 	f, err := s.pool.Open(path)
 	c.Assert(err, IsNil)
-	contents, err := ioutil.ReadAll(f)
+	contents, err := io.ReadAll(f)
 	c.Assert(err, IsNil)
 	c.Check(len(contents), Equals, 2738)
 	c.Check(f.Close(), IsNil)
@@ -326,7 +357,11 @@ func (s *PackagePoolSuite) TestLink(c *C) {
 	info, err := os.Stat(dstPath)
 	c.Assert(err, IsNil)
 	c.Check(info.Size(), Equals, int64(2738))
-	c.Check(info.Sys().(*syscall.Stat_t).Nlink > 2, Equals, true)
+	if isSameDevice(s) {
+		c.Check(info.Sys().(*syscall.Stat_t).Nlink > 2, Equals, true)
+	} else {
+		c.Check(info.Sys().(*syscall.Stat_t).Nlink == 2, Equals, true)
+	}
 }
 
 func (s *PackagePoolSuite) TestSymlink(c *C) {
@@ -340,7 +375,11 @@ func (s *PackagePoolSuite) TestSymlink(c *C) {
 	info, err := os.Stat(dstPath)
 	c.Assert(err, IsNil)
 	c.Check(info.Size(), Equals, int64(2738))
-	c.Check(info.Sys().(*syscall.Stat_t).Nlink > 2, Equals, true)
+	if isSameDevice(s) {
+		c.Check(info.Sys().(*syscall.Stat_t).Nlink > 2, Equals, true)
+	} else {
+		c.Check(info.Sys().(*syscall.Stat_t).Nlink == 1, Equals, true)
+	}
 
 	info, err = os.Lstat(dstPath)
 	c.Assert(err, IsNil)
