@@ -1,13 +1,12 @@
 package files
 
 import (
-	"io/ioutil"
 	"os"
 	"path/filepath"
 	"syscall"
 
-	"github.com/smira/aptly/aptly"
-	"github.com/smira/aptly/utils"
+	"github.com/aptly-dev/aptly/aptly"
+	"github.com/aptly-dev/aptly/utils"
 
 	. "gopkg.in/check.v1"
 )
@@ -129,6 +128,10 @@ func (s *PublishedStorageSuite) TestSymLink(c *C) {
 
 	exists, _ := s.storage.FileExists("ppa/dists/squeeze/InRelease")
 	c.Check(exists, Equals, true)
+
+	linkTarget, err := s.storage.ReadLink("ppa/dists/squeeze/InRelease")
+	c.Assert(err, IsNil)
+	c.Assert(linkTarget, Equals, "ppa/dists/squeeze/Release")
 }
 
 func (s *PublishedStorageSuite) TestHardLink(c *C) {
@@ -177,39 +180,40 @@ func (s *PublishedStorageSuite) TestRemove(c *C) {
 
 func (s *PublishedStorageSuite) TestLinkFromPool(c *C) {
 	tests := []struct {
-		prefix           string
-		component        string
-		sourcePath       string
-		poolDirectory    string
-		expectedFilename string
+		prefix             string
+		sourcePath         string
+		publishedDirectory string
+		expectedFilename   string
 	}{
 		{ // package name regular
-			prefix:           "",
-			component:        "main",
-			sourcePath:       "mars-invaders_1.03.deb",
-			poolDirectory:    "m/mars-invaders",
-			expectedFilename: "pool/main/m/mars-invaders/mars-invaders_1.03.deb",
+			prefix:             "",
+			sourcePath:         "mars-invaders_1.03.deb",
+			publishedDirectory: "pool/main/m/mars-invaders",
+			expectedFilename:   "pool/main/m/mars-invaders/mars-invaders_1.03.deb",
 		},
 		{ // lib-like filename
-			prefix:           "",
-			component:        "main",
-			sourcePath:       "libmars-invaders_1.03.deb",
-			poolDirectory:    "libm/libmars-invaders",
-			expectedFilename: "pool/main/libm/libmars-invaders/libmars-invaders_1.03.deb",
+			prefix:             "",
+			sourcePath:         "libmars-invaders_1.03.deb",
+			publishedDirectory: "pool/main/libm/libmars-invaders",
+			expectedFilename:   "pool/main/libm/libmars-invaders/libmars-invaders_1.03.deb",
 		},
 		{ // duplicate link, shouldn't panic
-			prefix:           "",
-			component:        "main",
-			sourcePath:       "mars-invaders_1.03.deb",
-			poolDirectory:    "m/mars-invaders",
-			expectedFilename: "pool/main/m/mars-invaders/mars-invaders_1.03.deb",
+			prefix:             "",
+			sourcePath:         "mars-invaders_1.03.deb",
+			publishedDirectory: "pool/main/m/mars-invaders",
+			expectedFilename:   "pool/main/m/mars-invaders/mars-invaders_1.03.deb",
 		},
 		{ // prefix & component
-			prefix:           "ppa",
-			component:        "contrib",
-			sourcePath:       "libmars-invaders_1.04.deb",
-			poolDirectory:    "libm/libmars-invaders",
-			expectedFilename: "pool/contrib/libm/libmars-invaders/libmars-invaders_1.04.deb",
+			prefix:             "ppa",
+			sourcePath:         "libmars-invaders_1.04.deb",
+			publishedDirectory: "pool/contrib/libm/libmars-invaders",
+			expectedFilename:   "pool/contrib/libm/libmars-invaders/libmars-invaders_1.04.deb",
+		},
+		{ // installer file
+			prefix:             "",
+			sourcePath:         "netboot/boot.img.gz",
+			publishedDirectory: "dists/jessie/non-free/installer-i386/current/images",
+			expectedFilename:   "dists/jessie/non-free/installer-i386/current/images/netboot/boot.img.gz",
 		},
 	}
 
@@ -217,7 +221,8 @@ func (s *PublishedStorageSuite) TestLinkFromPool(c *C) {
 
 	for _, t := range tests {
 		tmpPath := filepath.Join(c.MkDir(), t.sourcePath)
-		err := ioutil.WriteFile(tmpPath, []byte("Contents"), 0644)
+		_ = os.MkdirAll(filepath.Dir(tmpPath), 0777)
+		err := os.WriteFile(tmpPath, []byte("Contents"), 0644)
 		c.Assert(err, IsNil)
 
 		sourceChecksum, err := utils.ChecksumsForFile(tmpPath)
@@ -227,7 +232,7 @@ func (s *PublishedStorageSuite) TestLinkFromPool(c *C) {
 		c.Assert(err, IsNil)
 
 		// Test using hardlinks
-		err = s.storage.LinkFromPool(filepath.Join(t.prefix, "pool", t.component, t.poolDirectory), t.sourcePath, pool, srcPoolPath, sourceChecksum, false)
+		err = s.storage.LinkFromPool(t.prefix, t.publishedDirectory, t.sourcePath, pool, srcPoolPath, sourceChecksum, false)
 		c.Assert(err, IsNil)
 
 		st, err := os.Stat(filepath.Join(s.storage.rootPath, t.prefix, t.expectedFilename))
@@ -237,7 +242,7 @@ func (s *PublishedStorageSuite) TestLinkFromPool(c *C) {
 		c.Check(int(info.Nlink), Equals, 3)
 
 		// Test using symlinks
-		err = s.storageSymlink.LinkFromPool(filepath.Join(t.prefix, "pool", t.component, t.poolDirectory), t.sourcePath, pool, srcPoolPath, sourceChecksum, false)
+		err = s.storageSymlink.LinkFromPool(t.prefix, t.publishedDirectory, t.sourcePath, pool, srcPoolPath, sourceChecksum, false)
 		c.Assert(err, IsNil)
 
 		st, err = os.Lstat(filepath.Join(s.storageSymlink.rootPath, t.prefix, t.expectedFilename))
@@ -248,7 +253,7 @@ func (s *PublishedStorageSuite) TestLinkFromPool(c *C) {
 		c.Check(int(info.Mode&syscall.S_IFMT), Equals, int(syscall.S_IFLNK))
 
 		// Test using copy with checksum verification
-		err = s.storageCopy.LinkFromPool(filepath.Join(t.prefix, "pool", t.component, t.poolDirectory), t.sourcePath, pool, srcPoolPath, sourceChecksum, false)
+		err = s.storageCopy.LinkFromPool(t.prefix, t.publishedDirectory, t.sourcePath, pool, srcPoolPath, sourceChecksum, false)
 		c.Assert(err, IsNil)
 
 		st, err = os.Stat(filepath.Join(s.storageCopy.rootPath, t.prefix, t.expectedFilename))
@@ -258,7 +263,7 @@ func (s *PublishedStorageSuite) TestLinkFromPool(c *C) {
 		c.Check(int(info.Nlink), Equals, 1)
 
 		// Test using copy with size verification
-		err = s.storageCopySize.LinkFromPool(filepath.Join(t.prefix, "pool", t.component, t.poolDirectory), t.sourcePath, pool, srcPoolPath, sourceChecksum, false)
+		err = s.storageCopySize.LinkFromPool(t.prefix, t.publishedDirectory, t.sourcePath, pool, srcPoolPath, sourceChecksum, false)
 		c.Assert(err, IsNil)
 
 		st, err = os.Stat(filepath.Join(s.storageCopySize.rootPath, t.prefix, t.expectedFilename))
@@ -270,7 +275,7 @@ func (s *PublishedStorageSuite) TestLinkFromPool(c *C) {
 
 	// test linking files to duplicate final name
 	tmpPath := filepath.Join(c.MkDir(), "mars-invaders_1.03.deb")
-	err := ioutil.WriteFile(tmpPath, []byte("cONTENTS"), 0644)
+	err := os.WriteFile(tmpPath, []byte("cONTENTS"), 0644)
 	c.Assert(err, IsNil)
 
 	sourceChecksum, err := utils.ChecksumsForFile(tmpPath)
@@ -283,7 +288,7 @@ func (s *PublishedStorageSuite) TestLinkFromPool(c *C) {
 	c.Assert(err, IsNil)
 	nlinks := int(st.Sys().(*syscall.Stat_t).Nlink)
 
-	err = s.storage.LinkFromPool(filepath.Join("", "pool", "main", "m/mars-invaders"), "mars-invaders_1.03.deb", pool, srcPoolPath, sourceChecksum, false)
+	err = s.storage.LinkFromPool("", filepath.Join("pool", "main", "m/mars-invaders"), "mars-invaders_1.03.deb", pool, srcPoolPath, sourceChecksum, false)
 	c.Check(err, ErrorMatches, ".*file already exists and is different")
 
 	st, err = pool.Stat(srcPoolPath)
@@ -291,7 +296,7 @@ func (s *PublishedStorageSuite) TestLinkFromPool(c *C) {
 	c.Check(int(st.Sys().(*syscall.Stat_t).Nlink), Equals, nlinks)
 
 	// linking with force
-	err = s.storage.LinkFromPool(filepath.Join("", "pool", "main", "m/mars-invaders"), "mars-invaders_1.03.deb", pool, srcPoolPath, sourceChecksum, true)
+	err = s.storage.LinkFromPool("", filepath.Join("pool", "main", "m/mars-invaders"), "mars-invaders_1.03.deb", pool, srcPoolPath, sourceChecksum, true)
 	c.Check(err, IsNil)
 
 	st, err = pool.Stat(srcPoolPath)
@@ -299,20 +304,36 @@ func (s *PublishedStorageSuite) TestLinkFromPool(c *C) {
 	c.Check(int(st.Sys().(*syscall.Stat_t).Nlink), Equals, nlinks+1)
 
 	// Test using symlinks
-	err = s.storageSymlink.LinkFromPool(filepath.Join("", "pool", "main", "m/mars-invaders"), "mars-invaders_1.03.deb", pool, srcPoolPath, sourceChecksum, false)
+	err = s.storageSymlink.LinkFromPool("", filepath.Join("pool", "main", "m/mars-invaders"), "mars-invaders_1.03.deb", pool, srcPoolPath, sourceChecksum, false)
 	c.Check(err, ErrorMatches, ".*file already exists and is different")
 
-	err = s.storageSymlink.LinkFromPool(filepath.Join("", "pool", "main", "m/mars-invaders"), "mars-invaders_1.03.deb", pool, srcPoolPath, sourceChecksum, true)
+	err = s.storageSymlink.LinkFromPool("", filepath.Join("pool", "main", "m/mars-invaders"), "mars-invaders_1.03.deb", pool, srcPoolPath, sourceChecksum, true)
 	c.Check(err, IsNil)
 
 	// Test using copy with checksum verification
-	err = s.storageCopy.LinkFromPool(filepath.Join("", "pool", "main", "m/mars-invaders"), "mars-invaders_1.03.deb", pool, srcPoolPath, sourceChecksum, false)
+	err = s.storageCopy.LinkFromPool("", filepath.Join("pool", "main", "m/mars-invaders"), "mars-invaders_1.03.deb", pool, srcPoolPath, sourceChecksum, false)
 	c.Check(err, ErrorMatches, ".*file already exists and is different")
 
-	err = s.storageCopy.LinkFromPool(filepath.Join("", "pool", "main", "m/mars-invaders"), "mars-invaders_1.03.deb", pool, srcPoolPath, sourceChecksum, true)
+	err = s.storageCopy.LinkFromPool("", filepath.Join("pool", "main", "m/mars-invaders"), "mars-invaders_1.03.deb", pool, srcPoolPath, sourceChecksum, true)
 	c.Check(err, IsNil)
 
 	// Test using copy with size verification (this will NOT detect the difference)
-	err = s.storageCopySize.LinkFromPool(filepath.Join("", "pool", "main", "m/mars-invaders"), "mars-invaders_1.03.deb", pool, srcPoolPath, sourceChecksum, false)
+	err = s.storageCopySize.LinkFromPool("", filepath.Join("pool", "main", "m/mars-invaders"), "mars-invaders_1.03.deb", pool, srcPoolPath, sourceChecksum, false)
 	c.Check(err, IsNil)
 }
+
+func (s *PublishedStorageSuite) TestRootRemove(c *C) {
+	// Prevent deletion of the root directory by passing empty subpaths.
+
+	pwd := c.MkDir()
+
+	// Symlink
+	linkedDir := filepath.Join(pwd, "linkedDir")
+	_ = os.Symlink(s.root, linkedDir)
+	linkStorage := NewPublishedStorage(linkedDir, "", "")
+	c.Assert(func() { _ = linkStorage.Remove("") }, PanicMatches, "trying to remove empty path")
+
+	// Actual dir
+	dirStorage := NewPublishedStorage(pwd, "", "")
+	c.Assert(func() { _ = dirStorage.RemoveDirs("", nil) }, PanicMatches, "trying to remove the root directory")
+}
