@@ -6,7 +6,7 @@ import (
 	"io"
 	"net/url"
 
-	"github.com/smira/aptly/utils"
+	"github.com/aptly-dev/aptly/utils"
 
 	. "gopkg.in/check.v1"
 )
@@ -44,10 +44,12 @@ func (s *CompressionSuite) TestDownloadTryCompression(c *C) {
 	buf = make([]byte, 4)
 	d := NewFakeDownloader()
 	d.ExpectResponse("http://example.com/file.bz2", bzipData)
-	r, file, err := DownloadTryCompression(s.ctx, d, s.baseURL, "file", expectedChecksums, false, 1)
+	r, file, err := DownloadTryCompression(s.ctx, d, s.baseURL, "file", expectedChecksums, false)
 	c.Assert(err, IsNil)
-	defer file.Close()
-	io.ReadFull(r, buf)
+	defer func() {
+		_ = file.Close()
+	}()
+	_, _ = io.ReadFull(r, buf)
 	c.Assert(string(buf), Equals, rawData)
 	c.Assert(d.Empty(), Equals, true)
 
@@ -56,10 +58,12 @@ func (s *CompressionSuite) TestDownloadTryCompression(c *C) {
 	d = NewFakeDownloader()
 	d.ExpectError("http://example.com/file.bz2", &Error{Code: 404})
 	d.ExpectResponse("http://example.com/file.gz", gzipData)
-	r, file, err = DownloadTryCompression(s.ctx, d, s.baseURL, "file", expectedChecksums, false, 1)
+	r, file, err = DownloadTryCompression(s.ctx, d, s.baseURL, "file", expectedChecksums, false)
 	c.Assert(err, IsNil)
-	defer file.Close()
-	io.ReadFull(r, buf)
+	defer func() {
+		_ = file.Close()
+	}()
+	_, _ = io.ReadFull(r, buf)
 	c.Assert(string(buf), Equals, rawData)
 	c.Assert(d.Empty(), Equals, true)
 
@@ -69,10 +73,12 @@ func (s *CompressionSuite) TestDownloadTryCompression(c *C) {
 	d.ExpectError("http://example.com/file.bz2", &Error{Code: 404})
 	d.ExpectError("http://example.com/file.gz", &Error{Code: 404})
 	d.ExpectResponse("http://example.com/file.xz", xzData)
-	r, file, err = DownloadTryCompression(s.ctx, d, s.baseURL, "file", expectedChecksums, false, 1)
+	r, file, err = DownloadTryCompression(s.ctx, d, s.baseURL, "file", expectedChecksums, false)
 	c.Assert(err, IsNil)
-	defer file.Close()
-	io.ReadFull(r, buf)
+	defer func() {
+		_ = file.Close()
+	}()
+	_, _ = io.ReadFull(r, buf)
 	c.Assert(string(buf), Equals, rawData)
 	c.Assert(d.Empty(), Equals, true)
 
@@ -83,10 +89,12 @@ func (s *CompressionSuite) TestDownloadTryCompression(c *C) {
 	d.ExpectError("http://example.com/file.gz", &Error{Code: 404})
 	d.ExpectError("http://example.com/file.xz", &Error{Code: 404})
 	d.ExpectResponse("http://example.com/file", rawData)
-	r, file, err = DownloadTryCompression(s.ctx, d, s.baseURL, "file", expectedChecksums, false, 1)
+	r, file, err = DownloadTryCompression(s.ctx, d, s.baseURL, "file", expectedChecksums, false)
 	c.Assert(err, IsNil)
-	defer file.Close()
-	io.ReadFull(r, buf)
+	defer func() {
+		_ = file.Close()
+	}()
+	_, _ = io.ReadFull(r, buf)
 	c.Assert(string(buf), Equals, rawData)
 	c.Assert(d.Empty(), Equals, true)
 
@@ -94,7 +102,7 @@ func (s *CompressionSuite) TestDownloadTryCompression(c *C) {
 	d = NewFakeDownloader()
 	d.ExpectError("http://example.com/file.bz2", &Error{Code: 404})
 	d.ExpectResponse("http://example.com/file.gz", "x")
-	_, _, err = DownloadTryCompression(s.ctx, d, s.baseURL, "file", nil, true, 1)
+	_, _, err = DownloadTryCompression(s.ctx, d, s.baseURL, "file", nil, true)
 	c.Assert(err, ErrorMatches, "unexpected EOF")
 	c.Assert(d.Empty(), Equals, true)
 }
@@ -112,17 +120,19 @@ func (s *CompressionSuite) TestDownloadTryCompressionLongestSuffix(c *C) {
 	buf = make([]byte, 4)
 	d := NewFakeDownloader()
 	d.ExpectResponse("http://example.com/subdir/file.bz2", bzipData)
-	r, file, err := DownloadTryCompression(s.ctx, d, s.baseURL, "subdir/file", expectedChecksums, false, 1)
+	r, file, err := DownloadTryCompression(s.ctx, d, s.baseURL, "subdir/file", expectedChecksums, false)
 	c.Assert(err, IsNil)
-	defer file.Close()
-	io.ReadFull(r, buf)
+	defer func() {
+		_ = file.Close()
+	}()
+	_, _ = io.ReadFull(r, buf)
 	c.Assert(string(buf), Equals, rawData)
 	c.Assert(d.Empty(), Equals, true)
 }
 
 func (s *CompressionSuite) TestDownloadTryCompressionErrors(c *C) {
 	d := NewFakeDownloader()
-	_, _, err := DownloadTryCompression(s.ctx, d, s.baseURL, "file", nil, true, 1)
+	_, _, err := DownloadTryCompression(s.ctx, d, s.baseURL, "file", nil, true)
 	c.Assert(err, ErrorMatches, "unexpected request.*")
 
 	d = NewFakeDownloader()
@@ -130,7 +140,7 @@ func (s *CompressionSuite) TestDownloadTryCompressionErrors(c *C) {
 	d.ExpectError("http://example.com/file.gz", &Error{Code: 404})
 	d.ExpectError("http://example.com/file.xz", &Error{Code: 404})
 	d.ExpectError("http://example.com/file", errors.New("403"))
-	_, _, err = DownloadTryCompression(s.ctx, d, s.baseURL, "file", nil, true, 1)
+	_, _, err = DownloadTryCompression(s.ctx, d, s.baseURL, "file", nil, true)
 	c.Assert(err, ErrorMatches, "403")
 
 	d = NewFakeDownloader()
@@ -144,6 +154,6 @@ func (s *CompressionSuite) TestDownloadTryCompressionErrors(c *C) {
 		"file.xz":  {Size: 7},
 		"file":     {Size: 7},
 	}
-	_, _, err = DownloadTryCompression(s.ctx, d, s.baseURL, "file", expectedChecksums, false, 1)
+	_, _, err = DownloadTryCompression(s.ctx, d, s.baseURL, "file", expectedChecksums, false)
 	c.Assert(err, ErrorMatches, "checksums don't match.*")
 }
