@@ -2,21 +2,27 @@ package s3
 
 import (
 	"bytes"
-	"io/ioutil"
+	"context"
+	"io"
+	"os"
 	"path/filepath"
+	"sort"
+	"strings"
 
 	. "gopkg.in/check.v1"
 
-	"github.com/aws/aws-sdk-go/aws"
-	"github.com/aws/aws-sdk-go/service/s3"
+	"github.com/aws/aws-sdk-go-v2/aws"
+	"github.com/aws/aws-sdk-go-v2/service/s3"
+	"github.com/aws/aws-sdk-go-v2/service/s3/types"
 
-	"github.com/smira/aptly/files"
-	"github.com/smira/aptly/utils"
+	"github.com/aptly-dev/aptly/files"
+	"github.com/aptly-dev/aptly/utils"
 )
 
 type PublishedStorageSuite struct {
 	srv                      *Server
 	storage, prefixedStorage *PublishedStorage
+	noSuchBucketStorage      *PublishedStorage
 }
 
 var _ = Suite(&PublishedStorageSuite{})
@@ -27,12 +33,18 @@ func (s *PublishedStorageSuite) SetUpTest(c *C) {
 	c.Assert(err, IsNil)
 	c.Assert(s.srv, NotNil)
 
-	s.storage, err = NewPublishedStorage("aa", "bb", "", "test-1", s.srv.URL(), "test", "", "", "", "", false, true, false, false)
+	s.storage, err = NewPublishedStorage("aa", "bb", "", "test-1", s.srv.URL(), "test", "", "", "", "", false, true, false, false, false)
 	c.Assert(err, IsNil)
-	s.prefixedStorage, err = NewPublishedStorage("aa", "bb", "", "test-1", s.srv.URL(), "test", "", "lala", "", "", false, true, false, false)
+	s.prefixedStorage, err = NewPublishedStorage("aa", "bb", "", "test-1", s.srv.URL(), "test", "", "lala", "", "", false, true, false, false, false)
+	c.Assert(err, IsNil)
+	s.noSuchBucketStorage, err = NewPublishedStorage("aa", "bb", "", "test-1", s.srv.URL(), "no-bucket", "", "", "", "", false, true, false, false, false)
 	c.Assert(err, IsNil)
 
-	_, err = s.storage.s3.CreateBucket(&s3.CreateBucketInput{Bucket: aws.String("test")})
+	_, err = s.storage.s3.CreateBucket(context.TODO(), &s3.CreateBucketInput{
+		Bucket: aws.String("test"),
+		CreateBucketConfiguration: &types.CreateBucketConfiguration{
+			LocationConstraint: "test-1",
+		}})
 	c.Assert(err, IsNil)
 }
 
@@ -40,42 +52,53 @@ func (s *PublishedStorageSuite) TearDownTest(c *C) {
 	s.srv.Quit()
 }
 
+func (s *PublishedStorageSuite) checkGetRequestsEqual(c *C, prefix string, expectedGetRequestUris []string) {
+	getRequests := make([]string, 0, len(s.srv.Requests))
+	for _, r := range s.srv.Requests {
+		if r.Method == "GET" && strings.HasPrefix(r.RequestURI, prefix) {
+			getRequests = append(getRequests, r.RequestURI)
+		}
+	}
+	sort.Strings(getRequests)
+	c.Check(getRequests, DeepEquals, expectedGetRequestUris)
+}
+
 func (s *PublishedStorageSuite) GetFile(c *C, path string) []byte {
-	resp, err := s.storage.s3.GetObject(&s3.GetObjectInput{
+	resp, err := s.storage.s3.GetObject(context.TODO(), &s3.GetObjectInput{
 		Bucket: aws.String(s.storage.bucket),
 		Key:    aws.String(path),
 	})
 	c.Assert(err, IsNil)
 
-	body, err := ioutil.ReadAll(resp.Body)
-	resp.Body.Close()
+	body, err := io.ReadAll(resp.Body)
+	_ = resp.Body.Close()
 	c.Assert(err, IsNil)
 
 	return body
 }
 
 func (s *PublishedStorageSuite) AssertNoFile(c *C, path string) {
-	_, err := s.storage.s3.HeadObject(&s3.HeadObjectInput{
+	_, err := s.storage.s3.HeadObject(context.TODO(), &s3.HeadObjectInput{
 		Bucket: aws.String(s.storage.bucket),
 		Key:    aws.String(path),
 	})
-	c.Assert(err, ErrorMatches, ".*\n.*status code: 404.*")
+	c.Assert(err, ErrorMatches, ".*StatusCode: 404.*")
 }
 
 func (s *PublishedStorageSuite) PutFile(c *C, path string, data []byte) {
-	_, err := s.storage.s3.PutObject(&s3.PutObjectInput{
+	_, err := s.storage.s3.PutObject(context.TODO(), &s3.PutObjectInput{
 		Bucket:      aws.String(s.storage.bucket),
 		Key:         aws.String(path),
 		Body:        bytes.NewReader(data),
 		ContentType: aws.String("binary/octet-stream"),
-		ACL:         aws.String("private"),
+		ACL:         types.ObjectCannedACLPrivate,
 	})
 	c.Assert(err, IsNil)
 }
 
 func (s *PublishedStorageSuite) TestPutFile(c *C) {
 	dir := c.MkDir()
-	err := ioutil.WriteFile(filepath.Join(dir, "a"), []byte("welcome to s3!"), 0644)
+	err := os.WriteFile(filepath.Join(dir, "a"), []byte("welcome to s3!"), 0644)
 	c.Assert(err, IsNil)
 
 	err = s.storage.PutFile("a/b.txt", filepath.Join(dir, "a"))
@@ -93,7 +116,7 @@ func (s *PublishedStorageSuite) TestPutFilePlusWorkaround(c *C) {
 	s.storage.plusWorkaround = true
 
 	dir := c.MkDir()
-	err := ioutil.WriteFile(filepath.Join(dir, "a"), []byte("welcome to s3!"), 0644)
+	err := os.WriteFile(filepath.Join(dir, "a"), []byte("welcome to s3!"), 0644)
 	c.Assert(err, IsNil)
 
 	err = s.storage.PutFile("a/b+c.txt", filepath.Join(dir, "a"))
@@ -169,6 +192,11 @@ func (s *PublishedStorageSuite) TestRemove(c *C) {
 	s.AssertNoFile(c, "lala/xyz")
 }
 
+func (s *PublishedStorageSuite) TestRemoveNoSuchBucket(c *C) {
+	err := s.noSuchBucketStorage.Remove("a/b")
+	c.Check(err, IsNil)
+}
+
 func (s *PublishedStorageSuite) TestRemovePlusWorkaround(c *C) {
 	s.storage.plusWorkaround = true
 
@@ -217,6 +245,11 @@ func (s *PublishedStorageSuite) TestRemoveDirsPlusWorkaround(c *C) {
 	c.Check(list, DeepEquals, []string{"a", "b", "c", "lala/a", "lala/b", "lala/c", "testa"})
 }
 
+func (s *PublishedStorageSuite) TestRemoveDirsNoSuchBucket(c *C) {
+	err := s.noSuchBucketStorage.RemoveDirs("a/b", nil)
+	c.Check(err, ErrorMatches, ".*StatusCode: 404.*")
+}
+
 func (s *PublishedStorageSuite) TestRenameFile(c *C) {
 	c.Skip("copy not available in s3test")
 }
@@ -227,58 +260,139 @@ func (s *PublishedStorageSuite) TestLinkFromPool(c *C) {
 	cs := files.NewMockChecksumStorage()
 
 	tmpFile1 := filepath.Join(c.MkDir(), "mars-invaders_1.03.deb")
-	err := ioutil.WriteFile(tmpFile1, []byte("Contents"), 0644)
+	err := os.WriteFile(tmpFile1, []byte("Contents"), 0644)
 	c.Assert(err, IsNil)
 	cksum1 := utils.ChecksumInfo{MD5: "c1df1da7a1ce305a3b60af9d5733ac1d"}
 
 	tmpFile2 := filepath.Join(c.MkDir(), "mars-invaders_1.03.deb")
-	err = ioutil.WriteFile(tmpFile2, []byte("Spam"), 0644)
+	err = os.WriteFile(tmpFile2, []byte("Spam"), 0644)
 	c.Assert(err, IsNil)
 	cksum2 := utils.ChecksumInfo{MD5: "e9dfd31cc505d51fc26975250750deab"}
 
+	tmpFile3 := filepath.Join(c.MkDir(), "netboot/boot.img.gz")
+	_ = os.MkdirAll(filepath.Dir(tmpFile3), 0777)
+	err = os.WriteFile(tmpFile3, []byte("Contents"), 0644)
+	c.Assert(err, IsNil)
+	cksum3 := utils.ChecksumInfo{MD5: "c1df1da7a1ce305a3b60af9d5733ac1d"}
+
 	src1, err := pool.Import(tmpFile1, "mars-invaders_1.03.deb", &cksum1, true, cs)
 	c.Assert(err, IsNil)
 	src2, err := pool.Import(tmpFile2, "mars-invaders_1.03.deb", &cksum2, true, cs)
 	c.Assert(err, IsNil)
+	src3, err := pool.Import(tmpFile3, "netboot/boot.img.gz", &cksum3, true, cs)
+	c.Assert(err, IsNil)
 
 	// first link from pool
-	err = s.storage.LinkFromPool(filepath.Join("", "pool", "main", "m/mars-invaders"), "mars-invaders_1.03.deb", pool, src1, cksum1, false)
+	err = s.storage.LinkFromPool("", filepath.Join("pool", "main", "m/mars-invaders"), "mars-invaders_1.03.deb", pool, src1, cksum1, false)
 	c.Check(err, IsNil)
 
 	c.Check(s.GetFile(c, "pool/main/m/mars-invaders/mars-invaders_1.03.deb"), DeepEquals, []byte("Contents"))
 
 	// duplicate link from pool
-	err = s.storage.LinkFromPool(filepath.Join("", "pool", "main", "m/mars-invaders"), "mars-invaders_1.03.deb", pool, src1, cksum1, false)
+	err = s.storage.LinkFromPool("", filepath.Join("pool", "main", "m/mars-invaders"), "mars-invaders_1.03.deb", pool, src1, cksum1, false)
 	c.Check(err, IsNil)
 
 	c.Check(s.GetFile(c, "pool/main/m/mars-invaders/mars-invaders_1.03.deb"), DeepEquals, []byte("Contents"))
 
 	// link from pool with conflict
-	err = s.storage.LinkFromPool(filepath.Join("", "pool", "main", "m/mars-invaders"), "mars-invaders_1.03.deb", pool, src2, cksum2, false)
+	err = s.storage.LinkFromPool("", filepath.Join("pool", "main", "m/mars-invaders"), "mars-invaders_1.03.deb", pool, src2, cksum2, false)
 	c.Check(err, ErrorMatches, ".*file already exists and is different.*")
 
 	c.Check(s.GetFile(c, "pool/main/m/mars-invaders/mars-invaders_1.03.deb"), DeepEquals, []byte("Contents"))
 
 	// link from pool with conflict and force
-	err = s.storage.LinkFromPool(filepath.Join("", "pool", "main", "m/mars-invaders"), "mars-invaders_1.03.deb", pool, src2, cksum2, true)
+	err = s.storage.LinkFromPool("", filepath.Join("pool", "main", "m/mars-invaders"), "mars-invaders_1.03.deb", pool, src2, cksum2, true)
 	c.Check(err, IsNil)
 
 	c.Check(s.GetFile(c, "pool/main/m/mars-invaders/mars-invaders_1.03.deb"), DeepEquals, []byte("Spam"))
 
 	// for prefixed storage:
 	// first link from pool
-	err = s.prefixedStorage.LinkFromPool(filepath.Join("", "pool", "main", "m/mars-invaders"), "mars-invaders_1.03.deb", pool, src1, cksum1, false)
+	err = s.prefixedStorage.LinkFromPool("", filepath.Join("pool", "main", "m/mars-invaders"), "mars-invaders_1.03.deb", pool, src1, cksum1, false)
 	c.Check(err, IsNil)
 
 	// 2nd link from pool, providing wrong path for source file
 	//
 	// this test should check that file already exists in S3 and skip upload (which would fail if not skipped)
-	s.prefixedStorage.pathCache = nil
-	err = s.prefixedStorage.LinkFromPool(filepath.Join("", "pool", "main", "m/mars-invaders"), "mars-invaders_1.03.deb", pool, "wrong-looks-like-pathcache-doesnt-work", cksum1, false)
+	err = s.prefixedStorage.LinkFromPool("", filepath.Join("pool", "main", "m/mars-invaders"), "mars-invaders_1.03.deb", pool, "wrong-looks-like-pathcache-doesnt-work", cksum1, false)
 	c.Check(err, IsNil)
 
 	c.Check(s.GetFile(c, "lala/pool/main/m/mars-invaders/mars-invaders_1.03.deb"), DeepEquals, []byte("Contents"))
 
+	// link from pool with nested file name
+	err = s.storage.LinkFromPool("", "dists/jessie/non-free/installer-i386/current/images", "netboot/boot.img.gz", pool, src3, cksum3, false)
+	c.Check(err, IsNil)
+
+	c.Check(s.GetFile(c, "dists/jessie/non-free/installer-i386/current/images/netboot/boot.img.gz"), DeepEquals, []byte("Contents"))
+}
+
+func (s *PublishedStorageSuite) TestLinkFromPoolCache(c *C) {
+	root := c.MkDir()
+	pool := files.NewPackagePool(root, false)
+	cs := files.NewMockChecksumStorage()
+
+	tmpFile1 := filepath.Join(c.MkDir(), "mars-invaders_1.03.deb")
+	err := os.WriteFile(tmpFile1, []byte("Contents"), 0644)
+	c.Assert(err, IsNil)
+	cksum1 := utils.ChecksumInfo{MD5: "c1df1da7a1ce305a3b60af9d5733ac1d"}
+
+	src1, err := pool.Import(tmpFile1, "mars-invaders_1.03.deb", &cksum1, true, cs)
+	c.Assert(err, IsNil)
+
+	// Publish two packages at the same publish prefix
+	err = s.storage.LinkFromPool("", filepath.Join("pool", "a"), "mars-invaders_1.03.deb", pool, src1, cksum1, false)
+	c.Check(err, IsNil)
+
+	err = s.storage.LinkFromPool("", filepath.Join("pool", "b"), "mars-invaders_1.03.deb", pool, src1, cksum1, false)
+	c.Check(err, IsNil)
+
+	// Check only one listing request was done to the server
+	s.checkGetRequestsEqual(c, "/test?", []string{"/test?encryption=", "/test?encryption=", "/test?list-type=2&max-keys=1000&prefix=pool%2F"})
+
+	s.srv.Requests = nil
+	// Publish two packages at a different prefix
+	err = s.storage.LinkFromPool("publish-prefix", filepath.Join("pool", "a"), "mars-invaders_1.03.deb", pool, src1, cksum1, false)
+	c.Check(err, IsNil)
+
+	err = s.storage.LinkFromPool("publish-prefix", filepath.Join("pool", "b"), "mars-invaders_1.03.deb", pool, src1, cksum1, false)
+	c.Check(err, IsNil)
+
+	// Check no listing request was done to the server (pathCache is used)
+	s.checkGetRequestsEqual(c, "/test?", []string{})
+
+	s.srv.Requests = nil
+	// Publish two packages at a prefixed storage
+	err = s.prefixedStorage.LinkFromPool("", filepath.Join("pool", "a"), "mars-invaders_1.03.deb", pool, src1, cksum1, false)
+	c.Check(err, IsNil)
+
+	err = s.prefixedStorage.LinkFromPool("", filepath.Join("pool", "b"), "mars-invaders_1.03.deb", pool, src1, cksum1, false)
+	c.Check(err, IsNil)
+
+	// Check only one listing request was done to the server
+	s.checkGetRequestsEqual(c, "/test?", []string{
+		"/test?list-type=2&max-keys=1000&prefix=lala%2Fpool%2F",
+	})
+
+	// Publish two packages at a prefixed storage plus a publish prefix.
+	s.srv.Requests = nil
+	err = s.prefixedStorage.LinkFromPool("publish-prefix", filepath.Join("pool", "a"), "mars-invaders_1.03.deb", pool, src1, cksum1, false)
+	c.Check(err, IsNil)
+
+	err = s.prefixedStorage.LinkFromPool("publish-prefix", filepath.Join("pool", "b"), "mars-invaders_1.03.deb", pool, src1, cksum1, false)
+	c.Check(err, IsNil)
+
+	// Check no listing request was done to the server (pathCache is used)
+	s.checkGetRequestsEqual(c, "/test?", []string{})
+
+	// This step checks that files already exists in S3 and skip upload (which would fail if not skipped).
+	err = s.prefixedStorage.LinkFromPool("publish-prefix", filepath.Join("pool", "a"), "mars-invaders_1.03.deb", pool, "non-existent-file", cksum1, false)
+	c.Check(err, IsNil)
+	err = s.prefixedStorage.LinkFromPool("", filepath.Join("pool", "a"), "mars-invaders_1.03.deb", pool, "non-existent-file", cksum1, false)
+	c.Check(err, IsNil)
+	err = s.storage.LinkFromPool("publish-prefix", filepath.Join("pool", "a"), "mars-invaders_1.03.deb", pool, "non-existent-file", cksum1, false)
+	c.Check(err, IsNil)
+	err = s.storage.LinkFromPool("", filepath.Join("pool", "a"), "mars-invaders_1.03.deb", pool, "non-existent-file", cksum1, false)
+	c.Check(err, IsNil)
 }
 
 func (s *PublishedStorageSuite) TestSymLink(c *C) {
