@@ -3,11 +3,11 @@ package s3
 import (
 	"bytes"
 	"crypto/md5"
+	"encoding/base64"
 	"encoding/hex"
 	"encoding/xml"
 	"fmt"
 	"io"
-	"io/ioutil"
 	"log"
 	"net"
 	"net/http"
@@ -58,6 +58,12 @@ func (c *Config) send409Conflict() bool {
 	return false
 }
 
+// Request stores the method and URI of an HTTP request.
+type Request struct {
+	Method     string
+	RequestURI string
+}
+
 // Server is a fake S3 server for testing purposes.
 // All of the data for the server is kept in memory.
 type Server struct {
@@ -67,12 +73,13 @@ type Server struct {
 	mu       sync.Mutex
 	buckets  map[string]*bucket
 	config   *Config
+	// Requests holds a log of all requests received by the server.
+	Requests []Request
 }
 
 type bucket struct {
 	name    string
 	acl     string
-	ctime   time.Time
 	objects map[string]*object
 }
 
@@ -105,15 +112,15 @@ func NewServer(config *Config) (*Server, error) {
 		buckets:  make(map[string]*bucket),
 		config:   config,
 	}
-	go http.Serve(l, http.HandlerFunc(func(w http.ResponseWriter, req *http.Request) {
+	go func() { _ = http.Serve(l, http.HandlerFunc(func(w http.ResponseWriter, req *http.Request) {
 		srv.serveHTTP(w, req)
-	}))
+	})) }()
 	return srv, nil
 }
 
 // Quit closes down the server.
 func (srv *Server) Quit() {
-	srv.listener.Close()
+	_ = srv.listener.Close()
 }
 
 // URL returns a URL for the server.
@@ -132,7 +139,7 @@ func fatalError(code int, codeStr string, errf string, a ...interface{}) {
 // serveHTTP serves the S3 protocol.
 func (srv *Server) serveHTTP(w http.ResponseWriter, req *http.Request) {
 	// ignore error from ParseForm as it's usually spurious.
-	req.ParseForm()
+	_ = req.ParseForm()
 
 	srv.mu.Lock()
 	defer srv.mu.Unlock()
@@ -140,6 +147,7 @@ func (srv *Server) serveHTTP(w http.ResponseWriter, req *http.Request) {
 	if debug {
 		log.Printf("s3test %q %q", req.Method, req.URL)
 	}
+	srv.Requests = append(srv.Requests, Request{req.Method, req.RequestURI})
 	a := &action{
 		srv:   srv,
 		w:     w,
@@ -330,6 +338,10 @@ type Owner struct {
 	DisplayName string
 }
 
+type CommonPrefix struct {
+	Prefix string
+}
+
 // The ListResp type holds the results of a List bucket operation.
 type ListResp struct {
 	Name       string
@@ -344,7 +356,7 @@ type ListResp struct {
 	// http://goo.gl/YjQTc
 	IsTruncated    bool
 	Contents       []Key
-	CommonPrefixes []string `xml:">Prefix"`
+	CommonPrefixes []CommonPrefix
 }
 
 // The Key type represents an item stored in an S3 bucket.
@@ -371,7 +383,7 @@ func (r bucketResource) get(a *action) interface{} {
 	if s := a.req.Form.Get("max-keys"); s != "" {
 		i, err := strconv.Atoi(s)
 		if err != nil || i < 0 {
-			fatalError(400, "invalid value for max-keys: %q", s)
+			fatalError(400, "invalid value for max-keys", "%q", s)
 		}
 		maxKeys = i
 	}
@@ -403,7 +415,7 @@ func (r bucketResource) get(a *action) interface{} {
 		MaxKeys:   maxKeys,
 	}
 
-	var prefixes []string
+	var prefixes []CommonPrefix
 	for _, obj := range objs {
 		if !strings.HasPrefix(obj.name, prefix) {
 			continue
@@ -413,7 +425,7 @@ func (r bucketResource) get(a *action) interface{} {
 		if delimiter != "" {
 			if i := strings.Index(obj.name[len(prefix):], delimiter); i >= 0 {
 				name = obj.name[:len(prefix)+i+len(delimiter)]
-				if prefixes != nil && prefixes[len(prefixes)-1] == name {
+				if prefixes != nil && prefixes[len(prefixes)-1].Prefix == name {
 					continue
 				}
 				isPrefix = true
@@ -427,7 +439,7 @@ func (r bucketResource) get(a *action) interface{} {
 			break
 		}
 		if isPrefix {
-			prefixes = append(prefixes, name)
+			prefixes = append(prefixes, CommonPrefix{name})
 		} else {
 			// Contents contains only keys not found in CommonPrefixes
 			resp.Contents = append(resp.Contents, obj.s3Key())
@@ -528,7 +540,7 @@ func validBucketName(name string) bool {
 		return false
 	}
 	r := name[0]
-	if !(r >= '0' && r <= '9' || r >= 'a' && r <= 'z') {
+	if r < '0' || (r > '9' && r < 'a') || r > 'z' {
 		return false
 	}
 	for _, r := range name {
@@ -634,14 +646,14 @@ func (objr objectResource) put(a *action) interface{} {
 	var expectHash []byte
 	if c := a.req.Header.Get("Content-MD5"); c != "" {
 		var err error
-		expectHash, err = hex.DecodeString(c)
+		expectHash, err = base64.StdEncoding.DecodeString(c)
 		if err != nil || len(expectHash) != md5.Size {
 			fatalError(400, "InvalidDigest", "The Content-MD5 you specified was invalid")
 		}
 	}
 	sum := md5.New()
 	// TODO avoid holding lock while reading data.
-	data, err := ioutil.ReadAll(io.TeeReader(a.req.Body, sum))
+	data, err := io.ReadAll(io.TeeReader(a.req.Body, sum))
 	if err != nil {
 		fatalError(400, "TODO", "read error")
 	}
@@ -686,14 +698,14 @@ type CreateBucketConfiguration struct {
 func locationConstraint(a *action) string {
 	var body bytes.Buffer
 	if _, err := io.Copy(&body, a.req.Body); err != nil {
-		fatalError(400, "InvalidRequest", err.Error())
+		fatalError(400, "InvalidRequest", "%v", err.Error())
 	}
 	if body.Len() == 0 {
 		return ""
 	}
 	var loc CreateBucketConfiguration
 	if err := xml.NewDecoder(&body).Decode(&loc); err != nil {
-		fatalError(400, "InvalidRequest", err.Error())
+		fatalError(400, "InvalidRequest", "%v", err.Error())
 	}
 	return loc.LocationConstraint
 }
