@@ -31,6 +31,7 @@ class PublishUpdate1Test(BaseTest):
         self.check_exists('public/dists/maverick/main/binary-i386/Packages')
         self.check_exists('public/dists/maverick/main/binary-i386/Packages.gz')
         self.check_exists('public/dists/maverick/main/binary-i386/Packages.bz2')
+        self.check_exists('public/dists/maverick/Contents-i386.gz')
         self.check_exists('public/dists/maverick/main/Contents-i386.gz')
         self.check_exists('public/dists/maverick/main/source/Sources')
         self.check_exists('public/dists/maverick/main/source/Sources.gz')
@@ -48,9 +49,9 @@ class PublishUpdate1Test(BaseTest):
         self.check_file_contents('public/dists/maverick/main/binary-i386/Packages', 'binary', match_prepare=lambda s: "\n".join(sorted(s.split("\n"))))
 
         # verify signatures
-        self.run_cmd(["gpg", "--no-auto-check-trustdb", "--keyring", os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "files", "aptly.pub"),
+        self.run_cmd([self.gpgFinder.gpg, "--no-auto-check-trustdb", "--keyring", os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "files", "aptly.pub"),
                       "--verify", os.path.join(os.environ["HOME"], ".aptly", 'public/dists/maverick/InRelease')])
-        self.run_cmd(["gpg", "--no-auto-check-trustdb",  "--keyring", os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "files", "aptly.pub"),
+        self.run_cmd([self.gpgFinder.gpg, "--no-auto-check-trustdb",  "--keyring", os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "files", "aptly.pub"),
                       "--verify", os.path.join(os.environ["HOME"], ".aptly", 'public/dists/maverick/Release.gpg'),
                       os.path.join(os.environ["HOME"], ".aptly", 'public/dists/maverick/Release')])
 
@@ -60,6 +61,10 @@ class PublishUpdate1Test(BaseTest):
         pathsSeen = set()
         for l in release:
             fileHash, fileSize, path = l.split()
+            if "Contents" in path and not path.endswith(".gz"):
+                # "Contents" are present in index, but not really written to disk
+                continue
+
             pathsSeen.add(path)
 
             fileSize = int(fileSize)
@@ -77,14 +82,15 @@ class PublishUpdate1Test(BaseTest):
             else:
                 h = hashlib.sha512()
 
-            h.update(self.read_file(os.path.join('public/dists/maverick', path)))
+            h.update(self.read_file(os.path.join('public/dists/maverick', path), mode='b'))
 
             if h.hexdigest() != fileHash:
                 raise Exception("file hash doesn't match for %s: %s != %s" % (path, fileHash, h.hexdigest()))
 
         if pathsSeen != set(['main/binary-i386/Packages', 'main/binary-i386/Packages.bz2', 'main/binary-i386/Packages.gz',
                              'main/source/Sources', 'main/source/Sources.gz', 'main/source/Sources.bz2',
-                             'main/binary-i386/Release', 'main/source/Release', 'main/Contents-i386.gz']):
+                             'main/binary-i386/Release', 'main/source/Release', 'main/Contents-i386.gz',
+                             'Contents-i386.gz']):
             raise Exception("path seen wrong: %r" % (pathsSeen, ))
 
 
@@ -169,39 +175,6 @@ class PublishUpdate3Test(BaseTest):
         self.check_exists('public/pool/main/b/boost-defaults/libboost-program-options-dev_1.49.0.1_i386.deb')
 
 
-class PublishUpdate4Test(BaseTest):
-    """
-    publish update: added some packages, but list of published archs doesn't change
-    """
-    fixtureCmds = [
-        "aptly repo create local-repo",
-        "aptly repo add local-repo ${files}/pyspi_0.6.1-1.3.dsc",
-        "aptly publish repo -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=maverick local-repo",
-        "aptly repo add local-repo ${files}/libboost-program-options-dev_1.49.0.1_i386.deb"
-    ]
-    runCmd = "aptly publish update -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec maverick"
-    gold_processor = BaseTest.expand_environ
-
-    def check(self):
-        super(PublishUpdate4Test, self).check()
-
-        self.check_exists('public/dists/maverick/InRelease')
-        self.check_exists('public/dists/maverick/Release')
-        self.check_exists('public/dists/maverick/Release.gpg')
-
-        self.check_not_exists('public/dists/maverick/main/binary-i386/Packages')
-        self.check_not_exists('public/dists/maverick/main/binary-i386/Packages.gz')
-        self.check_not_exists('public/dists/maverick/main/binary-i386/Packages.bz2')
-        self.check_exists('public/dists/maverick/main/source/Sources')
-        self.check_exists('public/dists/maverick/main/source/Sources.gz')
-        self.check_exists('public/dists/maverick/main/source/Sources.bz2')
-
-        self.check_exists('public/pool/main/p/pyspi/pyspi_0.6.1-1.3.dsc')
-        self.check_exists('public/pool/main/p/pyspi/pyspi_0.6.1-1.3.diff.gz')
-        self.check_exists('public/pool/main/p/pyspi/pyspi_0.6.1.orig.tar.gz')
-        self.check_not_exists('public/pool/main/b/boost-defaults/libboost-program-options-dev_1.49.0.1_i386.deb')
-
-
 class PublishUpdate5Test(BaseTest):
     """
     publish update: no such publish
@@ -210,20 +183,6 @@ class PublishUpdate5Test(BaseTest):
     expectedCode = 1
 
 
-class PublishUpdate6Test(BaseTest):
-    """
-    publish update: not a local repo
-    """
-    fixtureDB = True
-    fixturePool = True
-    fixtureCmds = [
-        "aptly snapshot create snap1 from mirror gnuplot-maverick",
-        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec snap1",
-    ]
-    runCmd = "aptly publish update maverick"
-    expectedCode = 1
-
-
 class PublishUpdate7Test(BaseTest):
     """
     publish update: multiple components, add some packages
@@ -370,6 +329,7 @@ class PublishUpdate12Test(BaseTest):
         self.check_exists('public/dists/maverick/main/binary-i386/Packages')
         self.check_exists('public/dists/maverick/main/binary-i386/Packages.gz')
         self.check_exists('public/dists/maverick/main/binary-i386/Packages.bz2')
+        self.check_exists('public/dists/maverick/Contents-i386.gz')
         self.check_exists('public/dists/maverick/main/Contents-i386.gz')
         self.check_exists('public/dists/maverick/main/source/Sources')
         self.check_exists('public/dists/maverick/main/source/Sources.gz')
@@ -387,9 +347,9 @@ class PublishUpdate12Test(BaseTest):
         self.check_file_contents('public/dists/maverick/main/binary-i386/Packages', 'binary', match_prepare=lambda s: "\n".join(sorted(s.split("\n"))))
 
         # verify signatures
-        self.run_cmd(["gpg", "--no-auto-check-trustdb", "--keyring", os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "files", "aptly.pub"),
+        self.run_cmd([self.gpgFinder.gpg, "--no-auto-check-trustdb", "--keyring", os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "files", "aptly.pub"),
                       "--verify", os.path.join(os.environ["HOME"], ".aptly", 'public/dists/maverick/InRelease')])
-        self.run_cmd(["gpg", "--no-auto-check-trustdb",  "--keyring", os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "files", "aptly.pub"),
+        self.run_cmd([self.gpgFinder.gpg, "--no-auto-check-trustdb",  "--keyring", os.path.join(os.path.dirname(inspect.getsourcefile(BaseTest)), "files", "aptly.pub"),
                       "--verify", os.path.join(os.environ["HOME"], ".aptly", 'public/dists/maverick/Release.gpg'),
                       os.path.join(os.environ["HOME"], ".aptly", 'public/dists/maverick/Release')])
 
@@ -399,6 +359,10 @@ class PublishUpdate12Test(BaseTest):
         pathsSeen = set()
         for l in release:
             fileHash, fileSize, path = l.split()
+            if "Contents" in path and not path.endswith(".gz"):
+                # "Contents" are present in index, but not really written to disk
+                continue
+
             pathsSeen.add(path)
 
             fileSize = int(fileSize)
@@ -416,12 +380,229 @@ class PublishUpdate12Test(BaseTest):
             else:
                 h = hashlib.sha512()
 
-            h.update(self.read_file(os.path.join('public/dists/maverick', path)))
+            h.update(self.read_file(os.path.join('public/dists/maverick', path), mode='b'))
 
             if h.hexdigest() != fileHash:
                 raise Exception("file hash doesn't match for %s: %s != %s" % (path, fileHash, h.hexdigest()))
 
         if pathsSeen != set(['main/binary-i386/Packages', 'main/binary-i386/Packages.bz2', 'main/binary-i386/Packages.gz',
                              'main/source/Sources', 'main/source/Sources.gz', 'main/source/Sources.bz2',
-                             'main/binary-i386/Release', 'main/source/Release', 'main/Contents-i386.gz']):
+                             'main/binary-i386/Release', 'main/source/Release', 'main/Contents-i386.gz',
+                             'Contents-i386.gz']):
             raise Exception("path seen wrong: %r" % (pathsSeen, ))
+
+
+class PublishUpdate13Test(BaseTest):
+    """
+    publish update: -skip-bz2
+    """
+    fixtureCmds = [
+        "aptly repo create local-repo",
+        "aptly repo add local-repo ${files}/",
+        "aptly publish repo -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=maverick -skip-bz2 local-repo",
+        "aptly repo remove local-repo pyspi"
+    ]
+    runCmd = "aptly publish update -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -skip-bz2 maverick"
+    gold_processor = BaseTest.expand_environ
+
+    def check(self):
+        super(PublishUpdate13Test, self).check()
+
+        self.check_exists('public/dists/maverick/InRelease')
+        self.check_exists('public/dists/maverick/Release')
+        self.check_exists('public/dists/maverick/Release.gpg')
+
+        self.check_exists('public/dists/maverick/main/binary-i386/Packages')
+        self.check_exists('public/dists/maverick/main/binary-i386/Packages.gz')
+        self.check_not_exists('public/dists/maverick/main/binary-i386/Packages.bz2')
+
+
+class PublishUpdate14Test(BaseTest):
+    """
+    publish update: -multi-dist
+    """
+    fixtureCmds = [
+        "aptly repo create local-repo",
+        "aptly repo add local-repo ${files}/",
+        "aptly publish repo -keyring=${files}/aptly.pub -distribution=bookworm local-repo",
+    ]
+    runCmd = "aptly publish update -keyring=${files}/aptly.pub -multi-dist bookworm"
+    gold_processor = BaseTest.expand_environ
+
+    def check(self):
+        super(PublishUpdate14Test, self).check()
+
+        self.check_exists('public/dists/bookworm/InRelease')
+        self.check_exists('public/dists/bookworm/Release')
+        self.check_exists('public/dists/bookworm/Release.gpg')
+
+        self.check_exists('public/dists/bookworm/main/binary-i386/Packages')
+        self.check_exists('public/dists/bookworm/main/binary-i386/Packages.gz')
+
+        self.check_exists('public/pool/bookworm/main/b/boost-defaults/libboost-program-options-dev_1.49.0.1_i386.deb')
+
+
+class PublishUpdate15Test(BaseTest):
+    """
+    publish update: source added
+    """
+    fixtureDB = True
+    fixturePool = True
+    fixtureCmds = [
+        "aptly snapshot create snap1 from mirror gnuplot-maverick",
+        "aptly snapshot create snap2 empty",
+        "aptly snapshot create snap3 empty",
+        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=maverick -architectures=i386 -component=main snap1",
+        "aptly publish source add -component=test,other-test maverick snap2 snap3"
+    ]
+    runCmd = "aptly publish update -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec maverick"
+
+    gold_processor = BaseTest.expand_environ
+
+    def check(self):
+        super(PublishUpdate15Test, self).check()
+        self.check_exists('public/dists/maverick/InRelease')
+        self.check_exists('public/dists/maverick/Release')
+        self.check_exists('public/dists/maverick/Release.gpg')
+
+        self.check_exists('public/dists/maverick/main/binary-i386/Packages')
+        self.check_exists('public/dists/maverick/main/binary-i386/Packages.gz')
+        self.check_exists('public/dists/maverick/main/binary-i386/Packages.bz2')
+
+        self.check_exists('public/dists/maverick/test/binary-i386/Packages')
+        self.check_exists('public/dists/maverick/test/binary-i386/Packages.gz')
+        self.check_exists('public/dists/maverick/test/binary-i386/Packages.bz2')
+
+        self.check_exists('public/dists/maverick/other-test/binary-i386/Packages')
+        self.check_exists('public/dists/maverick/other-test/binary-i386/Packages.gz')
+        self.check_exists('public/dists/maverick/other-test/binary-i386/Packages.bz2')
+
+        release = self.read_file('public/dists/maverick/Release').split('\n')
+        components = next((e.split(': ')[1] for e in release if e.startswith('Components')), None)
+        components = sorted(components.split(' '))
+        if ['main', 'other-test', 'test'] != components:
+            raise Exception("value of 'Components' in release file is '%s' and does not match '%s'." % (' '.join(components), 'main other-test test'))
+
+
+class PublishUpdate16Test(BaseTest):
+    """
+    publish update: source removed
+    """
+    fixtureDB = True
+    fixturePool = True
+    fixtureCmds = [
+        "aptly snapshot create snap1 from mirror gnuplot-maverick",
+        "aptly snapshot create snap2 empty",
+        "aptly snapshot create snap3 empty",
+        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=maverick -architectures=i386 -component=main,test,other-test snap1 snap2 snap3",
+        "aptly publish source remove -component=test,other-test maverick"
+    ]
+    runCmd = "aptly publish update -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec maverick"
+
+    gold_processor = BaseTest.expand_environ
+
+    def check(self):
+        super(PublishUpdate16Test, self).check()
+        self.check_exists('public/dists/maverick/InRelease')
+        self.check_exists('public/dists/maverick/Release')
+        self.check_exists('public/dists/maverick/Release.gpg')
+
+        self.check_exists('public/dists/maverick/main/binary-i386/Packages')
+        self.check_exists('public/dists/maverick/main/binary-i386/Packages.gz')
+        self.check_exists('public/dists/maverick/main/binary-i386/Packages.bz2')
+        self.check_exists('public/dists/maverick/main/Contents-i386.gz')
+
+        release = self.read_file('public/dists/maverick/Release').split('\n')
+        components = next((e.split(': ')[1] for e in release if e.startswith('Components')), None)
+        components = sorted(components.split(' '))
+        if ['main'] != components:
+            raise Exception("value of 'Components' in release file is '%s' and does not match '%s'." % (' '.join(components), 'main'))
+
+
+class PublishUpdate17Test(BaseTest):
+    """
+    publish update: source updated
+    """
+    fixtureDB = True
+    fixturePool = True
+    fixtureCmds = [
+        "aptly snapshot create snap1 from mirror gnuplot-maverick",
+        "aptly snapshot create snap2 empty",
+        "aptly snapshot create snap3 empty",
+        "aptly snapshot create snap4 from mirror gnuplot-maverick",
+        "aptly snapshot create snap5 from mirror gnuplot-maverick",
+        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=maverick -architectures=i386 -component=main,test,other-test snap1 snap2 snap3",
+        "aptly publish source update -component=test,other-test maverick snap4 snap5"
+    ]
+    runCmd = "aptly publish update -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec maverick"
+
+    gold_processor = BaseTest.expand_environ
+
+    def check(self):
+        super(PublishUpdate17Test, self).check()
+        self.check_exists('public/dists/maverick/InRelease')
+        self.check_exists('public/dists/maverick/Release')
+        self.check_exists('public/dists/maverick/Release.gpg')
+
+        self.check_exists('public/dists/maverick/main/binary-i386/Packages')
+        self.check_exists('public/dists/maverick/main/binary-i386/Packages.gz')
+        self.check_exists('public/dists/maverick/main/binary-i386/Packages.bz2')
+        self.check_exists('public/dists/maverick/main/Contents-i386.gz')
+
+        self.check_exists('public/dists/maverick/test/binary-i386/Packages')
+        self.check_exists('public/dists/maverick/test/binary-i386/Packages.gz')
+        self.check_exists('public/dists/maverick/test/binary-i386/Packages.bz2')
+        self.check_exists('public/dists/maverick/test/Contents-i386.gz')
+
+        self.check_exists('public/dists/maverick/other-test/binary-i386/Packages')
+        self.check_exists('public/dists/maverick/other-test/binary-i386/Packages.gz')
+        self.check_exists('public/dists/maverick/other-test/binary-i386/Packages.bz2')
+        self.check_exists('public/dists/maverick/other-test/Contents-i386.gz')
+
+        release = self.read_file('public/dists/maverick/Release').split('\n')
+        components = next((e.split(': ')[1] for e in release if e.startswith('Components')), None)
+        components = sorted(components.split(' '))
+        if ['main', 'other-test', 'test'] != components:
+            raise Exception("value of 'Components' in release file is '%s' and does not match '%s'." % (' '.join(components), 'main other-test test'))
+
+
+class PublishUpdate18Test(BaseTest):
+    """
+    publish update: source added, updated and removed
+    """
+    fixtureDB = True
+    fixturePool = True
+    fixtureCmds = [
+        "aptly snapshot create snap1 from mirror gnuplot-maverick",
+        "aptly snapshot create snap2 empty",
+        "aptly snapshot create snap3 from mirror gnuplot-maverick",
+        "aptly publish snapshot -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec -distribution=maverick -architectures=i386 -component=main,test snap1 snap2",
+        "aptly publish source remove -component=main maverick",
+        "aptly publish source update -component=test maverick snap3",
+        "aptly publish source add -component=other-test maverick snap1"
+    ]
+    runCmd = "aptly publish update -keyring=${files}/aptly.pub -secret-keyring=${files}/aptly.sec maverick"
+
+    gold_processor = BaseTest.expand_environ
+
+    def check(self):
+        super(PublishUpdate18Test, self).check()
+        self.check_exists('public/dists/maverick/InRelease')
+        self.check_exists('public/dists/maverick/Release')
+        self.check_exists('public/dists/maverick/Release.gpg')
+
+        self.check_exists('public/dists/maverick/test/binary-i386/Packages')
+        self.check_exists('public/dists/maverick/test/binary-i386/Packages.gz')
+        self.check_exists('public/dists/maverick/test/binary-i386/Packages.bz2')
+        self.check_exists('public/dists/maverick/test/Contents-i386.gz')
+
+        self.check_exists('public/dists/maverick/other-test/binary-i386/Packages')
+        self.check_exists('public/dists/maverick/other-test/binary-i386/Packages.gz')
+        self.check_exists('public/dists/maverick/other-test/binary-i386/Packages.bz2')
+        self.check_exists('public/dists/maverick/other-test/Contents-i386.gz')
+
+        release = self.read_file('public/dists/maverick/Release').split('\n')
+        components = next((e.split(': ')[1] for e in release if e.startswith('Components')), None)
+        components = sorted(components.split(' '))
+        if ['other-test', 'test'] != components:
+            raise Exception("value of 'Components' in release file is '%s' and does not match '%s'." % (' '.join(components), 'other-test test'))
